'''This module implements the actual finalconfig application.
'''

import os.path

import pygtk
import gtk
import dbus
import dbus.service
import dbus.mainloop.glib
import gobject

import finalconfig.session
import finalconfig.const
import finalconfig.client.async

class Application (dbus.service.Object):
 
    def __init__ (self, config, desc):
        self.sessions = []
        self.session_handle = 0
        if config is None or config=='':
            config = finalconfig.const.CONFIG_PATH
            directory = os.path.abspath (os.path.expanduser (
                finalconfig.const.CONFIG_DIR))
            if not os.path.isdir (directory):
                os.makedirs (directory)
        if desc is None or desc=='':
            desc = finalconfig.const.DESC_PATH  
        self.config = finalconfig.client.async.Config (desc, config,
            finalconfig.const.APP_NAME)
        self.config.add_listener (self)
        gobject.idle_add (self.started)
        
    def run (self):
        gtk.main()
        
    def started (self):
        dbus.mainloop.glib.DBusGMainLoop (set_as_default=True) 
        name = dbus.service.BusName ('de.zpages.FinalConfig', dbus.service.SessionBus())
        dbus.service.Object.__init__ (self, conn = dbus.SessionBus(),
            object_path = '/', bus_name = name)
        return False

    @dbus.service.method (dbus_interface='de.zpages.FinalConfig', in_signature='sss',
        out_signature='s')
    def start_session (self, config_path, desc_path, app_name):
        for s in self.sessions:
            if (s.config_path==config_path and s.desc_path==desc_path and
                s.app_name==app_name):
                    # session already running
                    s.highlight()
                    return ''
        handle = '/session/%s' % self.session_handle
        self.session_handle = self.session_handle + 1
        session = finalconfig.session.Session (self, str (config_path), str (desc_path),
            handle, app_name, self.config.config)
        self.sessions.append (session)
        session.show()
        return handle
                        
    def close (self, session):
        self.sessions.remove (session)
        if len (self.sessions) == 0:
            gtk.main_quit()
            
    def shutdown (self):
        for s in self.sessions:
            s.close_session()
        gtk.main_quit()

    def changed (self, session):
        if session.app_name==finalconfig.const.APP_NAME:
            # Make sure externally started dialogues for FinalConfig itself trigger the
            # callback.
            self.config._reply (session.config_path)

    def on_config_changed (self, config):
        for s in self.sessions:
            s.on_config_changed (config)

