'''This module implements helper functions for handling the config file.
'''

import xml.dom.minidom


def read_config (config_path):
    '''Read config from a file and return a dictionary with the config entries.
    
    Note: This function can be used on desc files too. In this case the returned values
    are the default values.'''
    document = xml.dom.minidom.parse (config_path)
    entries = document.getElementsByTagName ('entry')
    config = dict()
    for entry in entries:
        if entry.hasAttribute ('id'):
            key = entry.attributes['id'].value
            value_nodes = entry.getElementsByTagName ('value')
            value_nodes = [v for v in value_nodes if v.parentNode==entry]
            if len (value_nodes) == 1:
                value = ""
                for node in value_nodes[0].childNodes:
                    if node.nodeType == node.TEXT_NODE:
                        value = value + node.data
                config[key] = value
    return config

def write_config (config, config_path):
    '''Write config dictionary into a file.'''
    document = xml.dom.minidom.Document()
    root = document.createElement ('config')
    document.appendChild (root)
    for key, value in config.iteritems():
        element = document.createElement ('entry')
        element.setAttribute ('id', key)
        value_node = document.createElement ('value')
        value_node.appendChild (document.createTextNode (value))
        element.appendChild (value_node)
        root.appendChild (element)
    f = open (config_path, 'w')
    document.writexml (f)
    f.close()

